@@ -1,32 +1,17 @@
-# Fork.pm -- Parallel management
-# $Id: Forker.pm 50266 2008-01-29 19:36:35Z wsnyder $
-######################################################################
-#
-# This program is Copyright 2002-2008 by Wilson Snyder.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of either the GNU General Public License or the
-# Perl Artistic License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
+# See copyright, etc in below POD section.
 ######################################################################
 
 package Parallel::Forker;
 require 5.006;
-use Time::HiRes qw (usleep);
-use Proc::ProcessTable;
+use Carp qw(carp croak confess);
 use IO::File;
+use Time::HiRes qw(usleep);
 
 use Parallel::Forker::Process;
 use strict;
-use Carp;
 use vars qw($Debug $VERSION);
 
-$VERSION = '1.222';
+$VERSION = '1.232';
 
 ######################################################################
 #### CONSTRUCTOR
@@ -39,8 +24,8 @@ sub new {
 	_labels => {},		# List of process objects, keyed by label
 	_runable => {},		# Process objects runable now, keyed by id
 	_running => {},		# Process objects running now, keyed *PID*
-	_in_child => 0,		# In a child process, don't allow forking
 	_run_after_eqn => undef,# Equation to eval to determine if ready to launch
+	_parent_pid => $$,	# PID of initial process creating the forker
 	max_proc => undef,	# Number processes to launch, <1=any, +=that number
 	use_sig_child => undef,	# Default to not using SIGCHLD handler
 	@_
@@ -51,6 +36,11 @@ sub new {
 
 #### ACCESSORS
 
+sub in_parent {
+    my $self = shift;
+    return $self->{_parent_pid}==$$;
+}
+
 sub max_proc {
     my $self = shift;
     $self->{max_proc} = shift if $#_>=0;
@@ -73,6 +63,12 @@ sub running_sorted {
     return (sort {$a->{name} cmp $b->{name}} values %{$self->{_running}});
 }
 
+sub process {
+    my $self = shift;
+    confess "usage: \$fork->process(\$name)" unless scalar(@_) == 1;
+    return $self->{_processes}{$_[0]};
+}
+
 sub processes {
     my $self = shift;
     return (values %{$self->{_processes}});
@@ -83,6 +79,14 @@ sub processes_sorted {
     return (sort {$a->{name} cmp $b->{name}} values %{$self->{_processes}});
 }
 
+sub state_stats {
+    my $self = shift;
+    my %stats = (idle=>0, ready=>0, running=>0, runable=>0,
+		  done=>0, parerr=>0, reapable=>0);
+    map {$stats{$_->state}++} $self->processes;
+    return %stats;
+}
+
 #### METHODS
 
 sub schedule {
@@ -106,6 +110,18 @@ sub wait_all {
     };
 }
 
+sub reap_processes {
+    my $self = shift;
+
+    my @reaped;
+    foreach my $process ($self->processes) {
+	next unless $process->is_reapable;
+	$process->reap;
+ 	push @reaped, $process;
+    }
+    return @reaped;
+}
+
 sub is_any_left {
     my $self = shift;
     return 1 if ( (keys %{$self->{_runable}}) > 0 );
@@ -306,7 +322,7 @@ Parallel::Forker - Parallel job forking and management
    use Parallel::Forker;
    $Fork = new Parallel::Forker (use_sig_child=>1);
    $SIG{CHLD} = sub { Parallel::Forker::sig_child($Fork); };
-   $SIG{TERM} = sub { $Fork->kill_tree_all('TERM') if $Fork; die "Quitting...\n"; };
+   $SIG{TERM} = sub { $Fork->kill_tree_all('TERM') if $Fork && $Fork->in_parent; die "Quitting...\n"; };
 
    $Fork->schedule
       (run_on_start => sub {print "child work here...";},
@@ -365,13 +381,20 @@ Returns one or more Parallel::Forker::Process objects for the given name (one
 object returned) or label (one or more objects returned).  Returns undef if no
 processes are found.
 
+=item $self->in_parent
+
+Return true if and only if called from the parent process (the one that
+created the Forker object).
+
 =item $self->is_any_left
 
 Return true if any processes are running, or runnable (need to run).
 
 =item $self->kill_all (<signal>)
 
-Send a signal to all running children.
+Send a signal to all running children.  You probably want to call this only
+from the parent process that created the Parallel::Forker object, wrap the
+call in "if ($self->in_parent)."
 
 =item $self->kill_tree_all (<signal>)
 
@@ -383,7 +406,8 @@ Specify the maximum number of processes that the poll method will run at
 any one time.  Defaults to undef, which runs all possible jobs at once.
 Max_proc takes effect when you schedule processes and mark them "ready,"
 then rely on Parallel::Forker's poll method to move the processes from the
-ready state to the run state.  (You do not need to call run yourself.)
+ready state to the run state.  (You should not call ->run yourself, as this
+starts a new process immediately, ignoring max_proc.)
 
 =item $self->new (<parameters>)
 
@@ -413,6 +437,11 @@ See if any children need work, and service them.  Start up to max_proc
 processes that are "ready" by calling their run method.  Non-blocking;
 always returns immediately.
 
+=item $self->process (<process_name>)
+
+Return Parallel::Forker::Process object for the specified process name, or
+undef if none is found.  See also find_proc_name.
+
 =item $self->processes
 
 Return Parallel::Forker::Process objects for all processes.
@@ -425,6 +454,13 @@ Return Parallel::Forker::Process objects for all processes, sorted by name.
 
 Mark all processes as ready for scheduling.
 
+=item $self->reap_processes
+
+Reap all processes which have no other processes waiting for them, and the
+process is is_done or is_parerr.  Returns list of processes reaped.  This
+reclaims memory for when a large number of processes are being created,
+run, and destroyed.
+
 =item $self->running
 
 Return Parallel::Forker::Process objects for all processes that are
@@ -442,7 +478,7 @@ follows:
 
 Optional name to use in C<run_after> commands.  Unlike C<name>, this may be
 reused, in which case C<run_after> will wait on all commands with the given
-label.
+label.  Labels must contain only [a-zA-Z0-9_].
 
 =item name
 
@@ -491,6 +527,11 @@ C<use_sig_child> was called with a "true" value.  If there are multiple
 Parallel::Forker objects each of their C<sig_child> methods must be called
 in the C<$SIG{CHLD}> handler.
 
+=item $self->state_stats
+
+Return hash containing statistics with keys of state names, and values with
+number of processes in each state.
+
 =item $self->use_sig_child ( 0 | 1 )
 
 This should always be called with a 0 or 1.  If you install a C<$SIG{CHLD}>
@@ -513,11 +554,12 @@ Print a dump of the execution tree.
 =head1 DISTRIBUTION
 
 The latest version is available from CPAN and from
-L<http://www.veripool.com/>.
+L<http://www.veripool.org/>.
 
-Copyright 2002-2008 by Wilson Snyder.  This package is free software; you
+Copyright 2002-2010 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+Lesser General Public License Version 3 or the Perl Artistic License
+Version 2.0.
 
 =head1 AUTHORS
 
