@@ -1,32 +1,18 @@
-# Fork.pm -- Parallel management
-# $Id: Process.pm 50266 2008-01-29 19:36:35Z wsnyder $
-######################################################################
-#
-# This program is Copyright 2002-2008 by Wilson Snyder.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of either the GNU General Public License or the
-# Perl Artistic License.
-# 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-# 
+# See copyright, etc in below POD section.
 ######################################################################
 
 package Parallel::Forker::Process;
 require 5.006;
-use Time::HiRes qw (usleep);
-use Proc::ProcessTable;
+use Carp qw(carp croak confess);
 use IO::File;
+use POSIX qw(sys_wait_h :signal_h);
+use Proc::ProcessTable;
+use Scalar::Util qw(weaken);
 
 use strict;
-use Carp;
-use POSIX qw(sys_wait_h :signal_h);
 use vars qw($Debug $VERSION $HashId);
 
-$VERSION = '1.222';
+$VERSION = '1.232';
 
 $Debug = $Parallel::Forker::Debug;
 $HashId = 0;
@@ -40,6 +26,7 @@ sub _new {
 	_after_children => {},	# IDs that are waiting on this event
 	_after_parents => {},	# IDs that we need to wait for
 	_state => 'idle',	# 'idle', 'ready', 'runable', 'running', 'done', 'parerr'
+	_ref_count => 0,        # number of people depending on us
 	pid => undef,		# Pid # running as, undef=not running
 	run_after => [],	# Process objects that are prereqs
 	run_on_start => sub {confess "%Error: No run_on_start defined\n";},
@@ -54,6 +41,8 @@ sub _new {
     (!exists $self->{_forkref}{_processes}{$self->{name}})
 	or croak "%Error: Creating a new process under the same name as an existing process: $self->{name},";
     $self->{_forkref}{_processes}{$self->{name}} = $self;
+    weaken($self->{_forkref});
+
     if (defined $self->{label}) {
 	if (ref $self->{label}) {
 	    foreach my $label (@{$self->{label}}) {
@@ -80,12 +69,21 @@ sub pid { return $_[0]->{pid}; }
 sub status { return $_[0]->{status}; }   # Maybe undef
 sub status_ok { return defined $_[0]->{status} && $_[0]->{status}==0; }
 sub forkref { return $_[0]->{_forkref}; }
+
+sub state { return $_[0]->{_state}; }
 sub is_idle    { return $_[0]->{_state} eq 'idle'; }
 sub is_ready   { return $_[0]->{_state} eq 'ready'; }
 sub is_runable { return $_[0]->{_state} eq 'runable'; }
 sub is_running { return $_[0]->{_state} eq 'running'; }
 sub is_done    { return $_[0]->{_state} eq 'done'; }
 sub is_parerr  { return $_[0]->{_state} eq 'parerr'; }
+sub is_reapable {
+    my $self = shift;
+    return $self->{_ref_count} == 0 && ($self->is_done || $self->is_parerr);
+}
+
+sub reference { $_[0]->{_ref_count}++ }
+sub unreference { $_[0]->{_ref_count}-- }
 
 ##### METHODS
 
@@ -150,8 +148,12 @@ sub _calc_eqns {
 			$runable_eqn .= " (_ranok('$aftname'))";
 			$parerr_eqn  .= " (_ranfail('$aftname')||_parerr('$aftname'))";
 		    }
+
 		    $aftref->{_after_children}{$self->{name}} = $self;
 		    $self->{_after_parents}{$aftref->{name}} = $aftref;
+		    weaken($aftref->{_after_children}{$self->{name}});
+		    weaken($self->{_after_parents}{$aftref->{name}});
+
 		    my $apo = $flip_op; $apo ||= 'O' if $between_op eq '||';
 		    $apo ||= '&';  $apo='E' if $apo eq '!';
 		    $self->{_after_parents_op}{$aftref->{name}} = $apo;
@@ -193,6 +195,9 @@ sub ready {
 
     # Transition: idle -> 'ready'
     print "  FrkProc $self->{name} $self->{_state} -> ready\n" if $Debug;
+    if (not $self->is_ready) {
+        $_->reference for values %{$self->{_after_parents}};
+    }
     $self->{_state} = 'ready';
     $self->_calc_runable();
 }
@@ -227,7 +232,6 @@ sub run {
 	$self->{_forkref}{_running}{$self->{pid}} = $self;
 	delete $self->{_forkref}{_runable}{$self->{name}};
     } else {
-	$self->{_forkref}{_in_child} = 1;
 	$self->{run_on_start}->($self);
 	exit(0);	# Don't close anything
     }
@@ -242,6 +246,26 @@ sub run_after {
     return $self;   # So can chain commands
 }
 
+sub reap {
+    my $self = shift;
+
+    $self->is_reapable or croak "%Error: process is not reapable,";
+    delete $self->{_forkref}{_processes}{$self->{name}};
+    if (defined $self->{label}) {
+	if (ref $self->{label}) {
+	    foreach my $label (@{$self->{label}}) {
+		@{$self->{_forkref}{_labels}{$label}} =
+		    grep { $_->{name} ne $self->{name} }
+		@{$self->{_forkref}{_labels}{$label}};
+	    }
+	} else {
+            @{$self->{_forkref}{_labels}{$self->{label}}} =
+		grep { $_->{name} ne $self->{name} }
+	    @{$self->{_forkref}{_labels}{$self->{label}}};
+	}
+    }
+}
+
 use vars qw($_Calc_Runable_Fork);
 
 sub _calc_runable {
@@ -274,7 +298,8 @@ sub _calc_runable {
 	$self->{_state} = 'runable';  # No dependencies (yet) so can launch it
 	$self->{_forkref}{_runable}{$self->{name}} = $self;
     } elsif (&{$self->{_parerr_eqn}}) {
-	$self->parerr;
+ 	$_->unreference for values %{$self->{_after_parents}};
+ 	$self->parerr;
     }
 }
 
@@ -289,7 +314,7 @@ sub poll {
     my $got = waitpid ($self->{pid}, WNOHANG);
     if ($got!=0) {
 	if ($got>0) {
-	    $self->{status} = $?;	# convert wait return to status 
+	    $self->{status} = $?;	# convert wait return to status
 	} else {
 	    $self->{status} = undef;
 	    carp "%Warning: waitpid($self->{pid}) returned -1 instead of status; perhaps you're ignoring SIG{CHLD}?"
@@ -307,6 +332,7 @@ sub poll {
 	foreach my $ra (values %{$self->{_after_children}}) {
 	    $ra->_calc_runable();
 	}
+ 	$_->unreference for values %{$self->{_after_parents}};
 	# Done
 	return $self;
     }
@@ -385,7 +411,7 @@ sub _subprocesses {
     # All pids under the given parent
     # Used by testing module
     # Same function in Schedule::Load::_subprocesses
-    my $pt = new Proc::ProcessTable( 'cache_ttys' => 1); 
+    my $pt = new Proc::ProcessTable( 'cache_ttys' => 1);
     my %parent_pids;
     foreach my $p (@{$pt->table}) {
 	$parent_pids{$p->pid} = $p->ppid;
@@ -458,6 +484,10 @@ Returns true if the process is in the parent error state.
 
 Returns true if the process is in the ready state.
 
+=item is_reapable
+
+Returns true if the process is reapable (->reap may be called on it).
+
 =item is_runable
 
 Returns true if the process is in the runable state.
@@ -481,14 +511,14 @@ running.  If no signal is specified, send a SIGKILL (9).
 Send a signal to this child (and its subchildren) if it is running.  If no
 signal is specified, send a SIGKILL (9).
 
-=item name
-
-Return the name of the process.
-
 =item label
 
 Return the label of the process, if any, else undef.
 
+=item name
+
+Return the name of the process.
+
 =item pid
 
 Return the process ID if this job is running, else undef.
@@ -504,7 +534,14 @@ Mark this process as being ready for execution when all C<run_after>'s are
 ready and CPU resources permit.  When that occurs, run will be called on
 the process automatically.
 
-=item run 
+=item reap
+
+When the process has no other processes waiting for it, and the process is
+is_done or is_parerr, remove the data structures for it.  This reclaims
+memory for when a large number of processes are being created, run, and
+destroyed.
+
+=item run
 
 Unconditionally move the process to the "running" state and start it.
 
@@ -518,6 +555,13 @@ process after ANY processes exit, or after ALL exit (the default.)
 bad exit status.  ^ in front of a process indicates to run if that process
 succeeds OR fails.
 
+=item state
+
+Returns the name of the current state, 'idle', 'ready', 'runable',
+'running', 'done' or 'parerr'.  For forward compatibility, use the is_idle
+etc. methods instead of comparing this accessor's value to a constant
+string.
+
 =item status
 
 Return the exit status of this process if it has completed.  The exit
@@ -535,11 +579,12 @@ undef.
 =head1 DISTRIBUTION
 
 The latest version is available from CPAN and from
-L<http://www.veripool.com/>.
+L<http://www.veripool.org/>.
 
-Copyright 2002-2008 by Wilson Snyder.  This package is free software; you
+Copyright 2002-2010 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+Lesser General Public License Version 3 or the Perl Artistic License
+Version 2.0.
 
 =head1 AUTHORS
 
